@@ -2,11 +2,14 @@
 /**
  * Provides static methods for charset and locale safe string manipulation.
  *
- * Copyright 2003-2014 Horde LLC (http://www.horde.org/)
+ * Copyright 2003-2015 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
+ * @todo Split up in Horde_String_Multibyte for multibyte-safe methods and
+ *       Horde_String_Locale for locale-safe methods.
+ *
  * @author   Jan Schneider <jan@horde.org>
  * @category Horde
  * @license  http://www.horde.org/licenses/lgpl21 LGPL 2.1
@@ -19,14 +22,14 @@ class Horde_String
      *
      * @var array
      */
-    static protected $_lowers = array();
+    protected static $_lowers = array();
 
     /**
      * upper() cache.
      *
      * @var array
      */
-    static protected $_uppers = array();
+    protected static $_uppers = array();
 
     /**
      * Converts a string from one charset to another.
@@ -44,7 +47,7 @@ class Horde_String
      *
      * @return mixed  The converted input data.
      */
-    static public function convertCharset($input, $from, $to, $force = false)
+    public static function convertCharset($input, $from, $to, $force = false)
     {
         /* Don't bother converting numbers. */
         if (is_numeric($input)) {
@@ -103,7 +106,7 @@ class Horde_String
      *
      * @return string  The converted string.
      */
-    static protected function _convertCharset($input, $from, $to)
+    protected static function _convertCharset($input, $from, $to)
     {
         /* Use utf8_[en|de]code() if possible and if the string isn't too
          * large (less than 16 MB = 16 * 1024 * 1024 = 16777216 bytes) - these
@@ -175,7 +178,7 @@ class Horde_String
      *
      * @return string  The string with lowercase characters.
      */
-    static public function lower($string, $locale = false, $charset = null)
+    public static function lower($string, $locale = false, $charset = null)
     {
         if ($locale) {
             if (Horde_Util::extensionExists('mbstring')) {
@@ -211,7 +214,7 @@ class Horde_String
      *
      * @return string  The string with uppercase characters.
      */
-    static public function upper($string, $locale = false, $charset = null)
+    public static function upper($string, $locale = false, $charset = null)
     {
         if ($locale) {
             if (Horde_Util::extensionExists('mbstring')) {
@@ -247,7 +250,7 @@ class Horde_String
      *
      * @return string  The capitalized string.
      */
-    static public function ucfirst($string, $locale = false, $charset = null)
+    public static function ucfirst($string, $locale = false, $charset = null)
     {
         if ($locale) {
             if (is_null($charset)) {
@@ -277,7 +280,7 @@ class Horde_String
      *
      * @return string  The capitalized string.
      */
-    static public function ucwords($string, $locale = false, $charset = null)
+    public static function ucwords($string, $locale = false, $charset = null)
     {
         $words = preg_split('/(\s+)/', $string, -1, PREG_SPLIT_DELIM_CAPTURE);
         for ($i = 0, $c = count($words); $i < $c; $i += 2) {
@@ -298,17 +301,19 @@ class Horde_String
      *
      * @return string  The string's part.
      */
-    static public function substr($string, $start, $length = null,
+    public static function substr($string, $start, $length = null,
                                   $charset = 'UTF-8')
     {
         if (is_null($length)) {
             $length = self::length($string, $charset) - $start;
         }
 
-        if ($length == 0) {
+        if ($length === 0) {
             return '';
         }
 
+        $error = false;
+
         /* Try mbstring. */
         if (Horde_Util::extensionExists('mbstring')) {
             $ret = @mb_substr($string, $start, $length, self::_mbstringCharset($charset));
@@ -317,6 +322,7 @@ class Horde_String
             if (strlen($ret)) {
                 return $ret;
             }
+            $error = true;
         }
 
         /* Try iconv. */
@@ -327,9 +333,31 @@ class Horde_String
             if ($ret !== false) {
                 return $ret;
             }
+            $error = true;
         }
 
-        return substr($string, $start, $length);
+        /* Try intl. */
+        if (Horde_Util::extensionExists('intl')) {
+            $ret = self::convertCharset(
+                @grapheme_substr(
+                    self::convertCharset($string, $charset, 'UTF-8'),
+                    $start,
+                    $length
+                ),
+                'UTF-8',
+                $charset
+            );
+
+            /* grapheme_substr() returns false on failure. */
+            if ($ret !== false) {
+                return $ret;
+            }
+            $error = true;
+        }
+
+        return $error
+            ? ''
+            : substr($string, $start, $length);
     }
 
     /**
@@ -341,7 +369,7 @@ class Horde_String
      *
      * @return integer  The string's length.
      */
-    static public function length($string, $charset = 'UTF-8')
+    public static function length($string, $charset = 'UTF-8')
     {
         $charset = self::lower($charset);
 
@@ -355,6 +383,11 @@ class Horde_String
                 return $ret;
             }
         }
+        if (Horde_Util::extensionExists('intl')) {
+            return grapheme_strlen(
+                self::convertCharset($string, $charset, 'UTF-8')
+            );
+        }
 
         return strlen($string);
     }
@@ -370,11 +403,11 @@ class Horde_String
      *
      * @return integer  The position of first occurrence.
      */
-    static public function pos(
+    public static function pos(
         $haystack, $needle, $offset = 0, $charset = 'UTF-8'
     )
     {
-        return self::_pos($haystack, $needle, $offset, $charset, 'mb_strpos', 'strpos');
+        return self::_pos($haystack, $needle, $offset, $charset, 'strpos');
     }
 
     /**
@@ -390,11 +423,11 @@ class Horde_String
      *
      * @return integer  The position of first case-insensitive occurrence.
      */
-    static public function ipos(
+    public static function ipos(
         $haystack, $needle, $offset = 0, $charset = 'UTF-8'
     )
     {
-        return self::_pos($haystack, $needle, $offset, $charset, 'mb_stripos', 'stripos');
+        return self::_pos($haystack, $needle, $offset, $charset, 'stripos');
     }
 
     /**
@@ -408,11 +441,11 @@ class Horde_String
      *
      * @return integer  The position of last occurrence.
      */
-    static public function rpos(
+    public static function rpos(
         $haystack, $needle, $offset = 0, $charset = 'UTF-8'
     )
     {
-        return self::_pos($haystack, $needle, $offset, $charset, 'mb_strrpos', 'strrpos');
+        return self::_pos($haystack, $needle, $offset, $charset, 'strrpos');
     }
 
     /**
@@ -428,11 +461,11 @@ class Horde_String
      *
      * @return integer  The position of last case-insensitive occurrence.
      */
-    static public function ripos(
+    public static function ripos(
         $haystack, $needle, $offset = 0, $charset = 'UTF-8'
     )
     {
-        return self::_pos($haystack, $needle, $offset, $charset, 'mb_strripos', 'strripos');
+        return self::_pos($haystack, $needle, $offset, $charset, 'strripos');
     }
 
     /**
@@ -442,19 +475,36 @@ class Horde_String
      * @param string $needle    The string to search for.
      * @param integer $offset   Character in $haystack to start searching at.
      * @param string $charset   Charset of $needle.
-     * @param string $mb_func   mb_* function to use.
-     * @param string $func      PHP string fallback function to use.
+     * @param string $func      Function to use.
      *
      * @return integer  The position of occurrence.
      *
      */
-    static protected function _pos(
-        $haystack, $needle, $offset, $charset, $mb_func, $func
+    protected static function _pos(
+        $haystack, $needle, $offset, $charset, $func
     )
     {
         if (Horde_Util::extensionExists('mbstring')) {
             $track_errors = ini_set('track_errors', 1);
-            $ret = @$mb_func($haystack, $needle, $offset, self::_mbstringCharset($charset));
+            $ret = @call_user_func('mb_' . $func, $haystack, $needle, $offset, self::_mbstringCharset($charset));
+            ini_set('track_errors', $track_errors);
+            if (!isset($php_errormsg)) {
+                return $ret;
+            }
+        }
+
+        if (Horde_Util::extensionExists('intl')) {
+            $track_errors = ini_set('track_errors', 1);
+            $ret = self::convertCharset(
+                @call_user_func(
+                    'grapheme_' . $func,
+                    self::convertCharset($haystack, $charset, 'UTF-8'),
+                    self::convertCharset($needle, $charset, 'UTF-8'),
+                    $offset
+                ),
+                'UTF-8',
+                $charset
+            );
             ini_set('track_errors', $track_errors);
             if (!isset($php_errormsg)) {
                 return $ret;
@@ -479,7 +529,7 @@ class Horde_String
      *
      * @return string  The padded string.
      */
-    static public function pad($input, $length, $pad = ' ',
+    public static function pad($input, $length, $pad = ' ',
                                $type = STR_PAD_RIGHT, $charset = 'UTF-8')
     {
         $mb_length = self::length($input, $charset);
@@ -535,7 +585,7 @@ class Horde_String
      *
      * @return string  String containing the wrapped text.
      */
-    static public function wordwrap($string, $width = 75, $break = "\n",
+    public static function wordwrap($string, $width = 75, $break = "\n",
                                     $cut = false, $line_folding = false)
     {
         $wrapped = '';
@@ -603,7 +653,7 @@ class Horde_String
      *
      * @return string  String containing the wrapped text.
      */
-    static public function wrap($text, $length = 80, $break_char = "\n",
+    public static function wrap($text, $length = 80, $break_char = "\n",
                                 $quote = false)
     {
         $paragraphs = array();
@@ -635,7 +685,7 @@ class Horde_String
      *
      * @return string  The truncated string, if longer than $length.
      */
-    static public function truncate($text, $length = 100)
+    public static function truncate($text, $length = 100)
     {
         return (self::length($text) > $length)
             ? rtrim(self::substr($text, 0, $length - 3)) . '...'
@@ -651,7 +701,7 @@ class Horde_String
      *
      * @return string  The abbreviated string, if longer than $length.
      */
-    static public function abbreviate($text, $length = 20)
+    public static function abbreviate($text, $length = 20)
     {
         return (self::length($text) > $length)
             ? rtrim(self::substr($text, 0, round(($length - 3) / 2))) . '...' . ltrim(self::substr($text, (($length - 3) / 2) * -1))
@@ -666,7 +716,7 @@ class Horde_String
      *
      * @return string  The start of $str1 and $str2 that is identical in both.
      */
-    static public function common($str1, $str2)
+    public static function common($str1, $str2)
     {
         for ($result = '', $i = 0;
              isset($str1[$i]) && isset($str2[$i]) && $str1[$i] == $str2[$i];
@@ -685,7 +735,7 @@ class Horde_String
      *
      * @return boolean  True if the parameter was alphabetic only.
      */
-    static public function isAlpha($string, $charset)
+    public static function isAlpha($string, $charset)
     {
         if (!Horde_Util::extensionExists('mbstring')) {
             return ctype_alpha($string);
@@ -714,7 +764,7 @@ class Horde_String
      *
      * @return boolean  True if the parameter was lowercase.
      */
-    static public function isLower($string, $charset)
+    public static function isLower($string, $charset)
     {
         return ((self::lower($string, true, $charset) === $string) &&
                 self::isAlpha($string, $charset));
@@ -729,7 +779,7 @@ class Horde_String
      *
      * @return boolean  True if the parameter was uppercase.
      */
-    static public function isUpper($string, $charset)
+    public static function isUpper($string, $charset)
     {
         return ((self::upper($string, true, $charset) === $string) &&
                 self::isAlpha($string, $charset));
@@ -745,7 +795,7 @@ class Horde_String
      *
      * @return array  The matches array from the first regex that matches.
      */
-    static public function regexMatch($text, $regex, $charset = null)
+    public static function regexMatch($text, $regex, $charset = null)
     {
         if (!empty($charset)) {
             $regex = self::convertCharset($regex, $charset, 'utf-8');
@@ -773,7 +823,7 @@ class Horde_String
      *
      * @return boolean  True if valid UTF-8.
      */
-    static public function validUtf8($text)
+    public static function validUtf8($text)
     {
         $text = strval($text);
 
@@ -817,7 +867,7 @@ class Horde_String
      *
      * @return string  The charset to use with mbstring functions.
      */
-    static protected function _mbstringCharset($charset)
+    protected static function _mbstringCharset($charset)
     {
         /* mbstring functions do not handle the 'ks_c_5601-1987' &
          * 'ks_c_5601-1989' charsets. However, these charsets are used, for
@@ -836,7 +886,7 @@ class Horde_String
      *
      * @return string  Stripped string (UTF-8).
      */
-    static public function trimUtf8Bom($str)
+    public static function trimUtf8Bom($str)
     {
         return (substr($str, 0, 3) == pack('CCC', 239, 187, 191))
             ? substr($str, 3)
