@@ -2,7 +2,7 @@
 /**
  * The Horde_Util:: class provides generally useful methods.
  *
- * Copyright 1999-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 1999-2015 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -22,7 +22,7 @@ class Horde_Util
      *
      * @var array
      */
-    static public $patterns = array(
+    public static $patterns = array(
         "\x55", "\xaa", "\x92\x49\x24", "\x49\x24\x92", "\x24\x92\x49",
         "\x00", "\x11", "\x22", "\x33", "\x44", "\x55", "\x66", "\x77",
         "\x88", "\x99", "\xaa", "\xbb", "\xcc", "\xdd", "\xee", "\xff",
@@ -35,16 +35,15 @@ class Horde_Util
      *
      * @var boolean
      */
-    static protected $_magicquotes = null;
+    protected static $_magicquotes = null;
 
     /**
-     * TODO
+     * Data used to determine shutdown deletion.
      *
      * @var array
      */
-    static protected $_shutdowndata = array(
-        'dirs' => array(),
-        'files' => array(),
+    protected static $_shutdowndata = array(
+        'paths' => array(),
         'secure' => array()
     );
 
@@ -53,14 +52,14 @@ class Horde_Util
      *
      * @var boolean
      */
-    static protected $_shutdownreg = false;
+    protected static $_shutdownreg = false;
 
     /**
      * Cache for extensionExists().
      *
      * @var array
      */
-    static protected $_cache = array();
+    protected static $_cache = array();
 
     /**
      * Checks to see if a value has been set by the script and not by GET,
@@ -74,7 +73,7 @@ class Horde_Util
      * @return mixed  $default if the var is in user input or not present,
      *                the variable value otherwise.
      */
-    static public function nonInputVar($varname, $default = null)
+    public static function nonInputVar($varname, $default = null)
     {
         return (isset($_GET[$varname]) || isset($_POST[$varname]) || isset($_COOKIE[$varname]))
             ? $default
@@ -88,7 +87,7 @@ class Horde_Util
      *
      * @return string  The hidden form input, if needed/requested.
      */
-    static public function formInput($append_session = 0)
+    public static function formInput($append_session = 0)
     {
         return (($append_session == 1) || !isset($_COOKIE[session_name()]))
             ? '<input type="hidden" name="' . htmlspecialchars(session_name()) . '" value="' . htmlspecialchars(session_id()) . "\" />\n"
@@ -100,7 +99,7 @@ class Horde_Util
      *
      * @param boolean $append_session  0 = only if needed, 1 = always.
      */
-    static public function pformInput($append_session = 0)
+    public static function pformInput($append_session = 0)
     {
         echo self::formInput($append_session);
     }
@@ -112,7 +111,7 @@ class Horde_Util
      *
      * @return mixed  $var, minus any magic quotes.
      */
-    static public function dispelMagicQuotes($var)
+    public static function dispelMagicQuotes($var)
     {
         if (is_null(self::$_magicquotes)) {
             self::$_magicquotes = get_magic_quotes_gpc();
@@ -139,7 +138,7 @@ class Horde_Util
      *
      * @return string  The cleaned form variable, or $default.
      */
-    static public function getFormData($var, $default = null)
+    public static function getFormData($var, $default = null)
     {
         return (($val = self::getPost($var)) !== null)
             ? $val
@@ -156,7 +155,7 @@ class Horde_Util
      *
      * @return string  The cleaned form variable, or $default.
      */
-    static public function getGet($var, $default = null)
+    public static function getGet($var, $default = null)
     {
         return (isset($_GET[$var]))
             ? self::dispelMagicQuotes($_GET[$var])
@@ -173,7 +172,7 @@ class Horde_Util
      *
      * @return string  The cleaned form variable, or $default.
      */
-    static public function getPost($var, $default = null)
+    public static function getPost($var, $default = null)
     {
         return (isset($_POST[$var]))
             ? self::dispelMagicQuotes($_POST[$var])
@@ -194,7 +193,7 @@ class Horde_Util
      * @return string   Returns the full path-name to the temporary file.
      *                  Returns false if a temp file could not be created.
      */
-    static public function getTempFile($prefix = '', $delete = true, $dir = '',
+    public static function getTempFile($prefix = '', $delete = true, $dir = '',
                                        $secure = false)
     {
         $tempDir = (empty($dir) || !is_dir($dir))
@@ -231,7 +230,7 @@ class Horde_Util
      * @return string   Returns the full path-name to the temporary file.
      *                  Returns false if a temporary file could not be created.
      */
-    static public function getTempFileWithExtension($extension = '.tmp',
+    public static function getTempFileWithExtension($extension = '.tmp',
                                                     $prefix = '',
                                                     $delete = true, $dir = '',
                                                     $secure = false)
@@ -292,7 +291,7 @@ class Horde_Util
      * @return string  The pathname to the new temporary directory.
      *                 Returns false if directory not created.
      */
-    static public function createTempDir($delete = true, $temp_dir = null)
+    public static function createTempDir($delete = true, $temp_dir = null)
     {
         if (is_null($temp_dir)) {
             $temp_dir = sys_get_temp_dir();
@@ -330,7 +329,7 @@ class Horde_Util
      *
      * @return string  The canonicalized file path.
      */
-    static public function realPath($path)
+    public static function realPath($path)
     {
         /* Standardize on UNIX directory separators. */
         if (!strncasecmp(PHP_OS, 'WIN', 3)) {
@@ -386,7 +385,7 @@ class Horde_Util
      * @param boolean $secure    If deleting file, should we securely delete
      *                           the file?
      */
-    static public function deleteAtShutdown($filename, $register = true,
+    public static function deleteAtShutdown($filename, $register = true,
                                             $secure = false)
     {
         /* Initialization of variables and shutdown functions. */
@@ -397,17 +396,12 @@ class Horde_Util
 
         $ptr = &self::$_shutdowndata;
         if ($register) {
-            if (@is_dir($filename)) {
-                $ptr['dirs'][$filename] = true;
-            } else {
-                $ptr['files'][$filename] = true;
-            }
-
+            $ptr['paths'][$filename] = true;
             if ($secure) {
                 $ptr['secure'][$filename] = true;
             }
         } else {
-            unset($ptr['dirs'][$filename], $ptr['files'][$filename], $ptr['secure'][$filename]);
+            unset($ptr['paths'][$filename], $ptr['secure'][$filename]);
         }
     }
 
@@ -421,43 +415,62 @@ class Horde_Util
      * Contains code from gpg_functions.php.
      * Copyright 2002-2003 Braverock Ventures
      */
-    static public function shutdown()
+    public static function shutdown()
     {
         $ptr = &self::$_shutdowndata;
 
-        foreach ($ptr['files'] as $file => $val) {
-            /* Delete files */
-            if ($val && file_exists($file)) {
-                /* Should we securely delete the file by overwriting the data
-                   with a random string? */
-                if (isset($ptr['secure'][$file])) {
-                    $filesize = filesize($file);
-                    $fp = fopen($file, 'r+');
-                    foreach (self::$patterns as $pattern) {
-                        $pattern = substr(str_repeat($pattern, floor($filesize / strlen($pattern)) + 1), 0, $filesize);
-                        fwrite($fp, $pattern);
-                        fseek($fp, 0);
-                    }
-                    fclose($fp);
-                }
-                @unlink($file);
+        foreach (array_keys($ptr['paths']) as $val) {
+            if (@is_file($val)) {
+                self::_secureDelete($val);
+                continue;
             }
+
+            try {
+                $it = new RecursiveIteratorIterator(
+                    new RecursiveDirectoryIterator($val),
+                    RecursiveIteratorIterator::CHILD_FIRST
+                );
+            } catch (UnexpectedValueException $e) {
+                continue;
+            }
+
+            while ($it->valid()) {
+                if (!$it->isDot()) {
+                    if ($it->isDir()) {
+                        @rmdir($it->key());
+                    } elseif ($it->isFile()) {
+                        self::_secureDelete($it->key());
+                    } else {
+                        @unlink($it->key());
+                    }
+                }
+                $it->next();
+            }
+
+            @rmdir($val);
+        }
+    }
+
+    /**
+     * Securely delete the file by overwriting the data with a random
+     * string.
+     *
+     * @param string $file  Filename.
+     */
+    protected static function _secureDelete($file)
+    {
+        if (isset($ptr['secure'][$file])) {
+            $filesize = filesize($file);
+            $fp = fopen($file, 'r+');
+            foreach (self::$patterns as $pattern) {
+                $pattern = substr(str_repeat($pattern, floor($filesize / strlen($pattern)) + 1), 0, $filesize);
+                fwrite($fp, $pattern);
+                fseek($fp, 0);
+            }
+            fclose($fp);
         }
 
-        foreach ($ptr['dirs'] as $dir => $val) {
-            /* Delete directories */
-            if ($val && file_exists($dir)) {
-                /* Make sure directory is empty. */
-                $dir_class = dir($dir);
-                while (false !== ($entry = $dir_class->read())) {
-                    if ($entry != '.' && $entry != '..') {
-                        @unlink($dir . '/' . $entry);
-                    }
-                }
-                $dir_class->close();
-                @rmdir($dir);
-            }
-        }
+        @unlink($file);
     }
 
     /**
@@ -467,7 +480,7 @@ class Horde_Util
      *
      * @return boolean  Is the extension loaded?
      */
-    static public function extensionExists($ext)
+    public static function extensionExists($ext)
     {
         if (!isset(self::$_cache[$ext])) {
             self::$_cache[$ext] = extension_loaded($ext);
@@ -486,7 +499,7 @@ class Horde_Util
      *                  True can mean that the extension was already loaded,
      *                  OR was loaded dynamically.
      */
-    static public function loadExtension($ext)
+    public static function loadExtension($ext)
     {
         /* If $ext is already loaded, our work is done. */
         if (self::extensionExists($ext)) {
@@ -530,7 +543,7 @@ class Horde_Util
      *
      * @return string  The PATH_INFO string.
      */
-    static public function getPathInfo()
+    public static function getPathInfo()
     {
         if (isset($_SERVER['PATH_INFO']) &&
             (strpos($_SERVER['SERVER_SOFTWARE'], 'lighttpd') === false)) {
