@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2009-2014 Horde LLC (http://www.horde.org/)
+ * Copyright 2009-2015 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
  * @category  Horde
- * @copyright 2009-2014 Horde LLC
+ * @copyright 2009-2015 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Util
  */
@@ -18,7 +18,7 @@
  * @author    Chuck Hagenbuch <chuck@horde.org>
  * @author    Michael Slusarz <slusarz@horde.org>
  * @category  Horde
- * @copyright 2009-2014 Horde LLC
+ * @copyright 2009-2015 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Util
  */
@@ -52,7 +52,7 @@ class Horde_Variables implements ArrayAccess, Countable, IteratorAggregate
      *
      * @return Horde_Variables  Variables object.
      */
-    static public function getDefaultVariables($sanitize = false)
+    public static function getDefaultVariables($sanitize = false)
     {
         return new self(null, $sanitize);
     }
