@@ -1,12 +1,12 @@
 <?php
 /**
- * Copyright 2014 Horde LLC (http://www.horde.org/)
+ * Copyright 2014-2015 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
  *
  * @category  Horde
- * @copyright 2014 Horde LLC
+ * @copyright 2014-2015 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Util
  */
@@ -16,24 +16,24 @@
  *
  * @author    Jan Schneider <jan@horde.org>
  * @category  Horde
- * @copyright 2014 Horde LLC
+ * @copyright 2014-2015 Horde LLC
  * @license   http://www.horde.org/licenses/lgpl21 LGPL 2.1
  * @package   Util
  * @since     2.4.0
  */
 class Horde_Util_Mock_Transliterate extends Horde_String_Transliterate
 {
-    static public function testIntl($str)
+    public static function testIntl($str)
     {
         return self::_intlToAscii($str);
     }
 
-    static public function testIconv($str)
+    public static function testIconv($str)
     {
         return self::_iconvToAscii($str);
     }
 
-    static public function testFallback($str)
+    public static function testFallback($str)
     {
         return self::_fallbackToAscii($str);
     }
